package SQ.Blocks.Indicators.ATRPercentRank;

import com.strategyquant.lib.*;
import com.strategyquant.datalib.*;
import com.strategyquant.tradinglib.*;
import SQ.Blocks.Indicators.MTATR.MTATR;
import SQ.Blocks.Indicators.ATR.ATR;
import com.strategyquant.tradinglib.simulator.Engines;

import SQ.Internal.IndicatorBlock;

/**
 * Indicator name as it will be displayed in UI, and its return type.
 * Possible return types:
 * ReturnTypes.Price - indicator is drawn on the price chart, like SMA, Bollinger Bands etc.
 * ReturnTypes.Number - indicator is drawn on separate chart, like CCI, RSI, MACD
 * ReturnTypes.PriceRange - indicator is price range, like ATR.
 */
@BuildingBlock(name="(ATRPR) ATR Percent Rank", display="ATR Percent Rank(@Chart@#ATRPeriod#,#Lenght#)[#Shift#]", returnType = ReturnTypes.Number)
@Help("ATRPercentRank help text")
@Indicator(oscillator=true, middleValue=50, min=0, max=100, step=0.1)
@ParameterSet(set="ATRPeriod=12,Lenght=24")
@ParameterSet(set="ATRPeriod=12,Lenght=48")
@ParameterSet(set="ATRPeriod=12,Lenght=96")
@ParameterSet(set="ATRPeriod=12,Lenght=120")
@ParameterSet(set="ATRPeriod=12,Lenght=10")
@ParameterSet(set="ATRPeriod=12,Lenght=20")
@ParameterSet(set="ATRPeriod=12,Lenght=40")
@ParameterSet(set="ATRPeriod=12,Lenght=60")
@ParameterSet(set="ATRPeriod=12,Lenght=120")
@ParameterSet(set="ATRPeriod=12,Lenght=200")
public class ATRPercentRank extends IndicatorBlock {

	@Parameter
	public ChartData Input;

	@Parameter(defaultValue="12", isPeriod=true, minValue=2, maxValue=1000, step=1)
	public int ATRPeriod;

	@Parameter(defaultValue="120", isPeriod=true, builderMinValue=2, builderMaxValue=500, minValue=2, maxValue=500, step=1)
	public int Lenght;

	@Output
	public DataSeries Value;

	//------------------------------------------------------------------------
	//------------------------------------------------------------------------
	//------------------------------------------------------------------------

	/**
	 * This method is called on every bar update and here the indicator value is computed.
	 *
	 * Unlike in MT4 you don't compute indicator values for multiple bars in a loop, 
	 * you need to compute value only for the latest (current) bar.
	 * Trading engine will take care of calling this method for every bar in the chart.
	 *
	 * Actual bar for which the indicator value is computed is stored in CurrentBar variable. 
	 * If 0, it means it is the very first bar of the chart.
	 */
	@Override
	protected void OnBarUpdate() throws TradingException {
		if(Engines.isTradestationEngine(Indicators.Engine)) {
			onBarUpdateTS();
		}
		else {
			
			onBarUpdateMT();
		}
	}

	protected void onBarUpdateMT() throws TradingException {

		// MT4/5 ATR
		MTATR indicator = Indicators.MTATR(Input, ATRPeriod);
		double atrValue = indicator.Value.getRounded(0,5);
		int atrRank = 0;
		for(int i = 1; i<= Lenght; i++){
			if(atrValue>indicator.Value.getRounded(i,5)){
				atrRank = atrRank+1;

			}
			
		}

		double atrPerRank = 100.0d*(double)atrRank/(double)Lenght;

		Value.set(0,atrPerRank);

		
		
  }	

  protected void onBarUpdateTS() throws TradingException {

		ATR indicator = Indicators.ATR(Input, ATRPeriod);
		double atrValue = indicator.Value.get(0);
		int atrRank = 0;
		for(int i = 1; i<= Lenght; i++){
			if(atrValue>indicator.Value.get(i)){
				atrRank = atrRank+1;

			}
		}

		double atrPerRank = 100.0d*(double)atrRank/(double)Lenght;

		Value.set(0,atrPerRank);
	  
	  
	  
	  
	  

  }


















}