package SQ.Blocks.Indicators.ATRTrailingStops;

import com.strategyquant.lib.*;
import com.strategyquant.datalib.*;
import com.strategyquant.tradinglib.*;
import SQ.Calculators.AverageCalculator;
import SQ.Internal.IndicatorBlock;
import SQ.Blocks.Indicators.MTATR.MTATR;
import SQ.Blocks.Indicators.ATR.ATR;
import com.strategyquant.tradinglib.simulator.Engines;

import SQ.Internal.IndicatorBlock;

/**
 * Indicator name as it will be displayed in UI, and its return type.
 * Possible return types:
 * ReturnTypes.Price - indicator is drawn on the price chart, like SMA, Bollinger Bands etc.
 * ReturnTypes.Number - indicator is drawn on separate chart, like CCI, RSI, MACD
 * ReturnTypes.PriceRange - indicator is price range, like ATR.
 */
@BuildingBlock(name="(ATS) ATR Trailing Stops", display="ATR Trailing Stops(@Chart@#ATRPeriod#,#ATRSmoothigPeriod#,#ATRMultiplier#,#ATRSmoothingMode#,#Method#)[#Shift#]", returnType = ReturnTypes.Price)
@Help("ATRTrailingStops help text")
public class ATRTrailingStops extends IndicatorBlock {

	@Parameter
	public ChartData Input;

	@Parameter(defaultValue="24", isPeriod=true, minValue=2, maxValue=1000, step=1)
	public int ATRPeriod;

	@Parameter(defaultValue="100", isPeriod=true, minValue=2, maxValue=1000, step=1)
	public int ATRSmoothigPeriod;

	@Parameter(defaultValue="2", isPeriod=false, minValue=0.1, maxValue=100, step=0.1)
	public double ATRMultiplier;

	@Parameter(name="ATRSmoothingMode", defaultValue="0")
	@Editor(type=Editors.Selection, values="Simple=0,Exponential=1,Smoothed=2,Linear weighted=3")
	public int ATRSmoothingMode;

	@Parameter(name="Method", defaultValue="0")
	@Editor(type=Editors.Selection, values="Close=0,High/Low=1,High/Low/Close=2")
	public int Method;


	@Output(name = "ATRTrailingStop", color = Colors.Blue)
	public DataSeries ATRTrailingStop;

	private AverageCalculator averageCalculator;

	//------------------------------------------------------------------------
	//------------------------------------------------------------------------
	//------------------------------------------------------------------------

	@Override
	protected void OnInit() throws TradingException {
		
		ATRSmoothingMode = SQUtils.fixAllowedRange(ATRSmoothingMode, 0, 3, 0);		
		averageCalculator = new AverageCalculator(ATRSmoothingMode, ATRSmoothigPeriod);
	}


	@Override
	protected void OnBarUpdate() throws TradingException {
      double indiValue;

		if (CurrentBar < ATRPeriod) {

			ATRTrailingStop.set(0,0);
						
		} else {

			MTATR indicator = Indicators.MTATR(Input, ATRPeriod);
  	  		double atrValue = indicator.Value.get(0);
			averageCalculator.onBarUpdate(atrValue, getCurrentBar());		
			//double atrMultiplied =  * ATRMultiplier;
			double atrMultiplied = averageCalculator.getValue();
			
			double Stop = atrMultiplied*ATRMultiplier;
			double ATRTrailingStopVar = 0.0;

		if(Method ==0){

			if (Input.Close.get(0)>ATRTrailingStop.get(1) &&  Input.Close.get(1)>ATRTrailingStop.get(1)){
				
				ATRTrailingStopVar  = Math.max(ATRTrailingStop.get(1), Input.Close.get(0)-Stop);

				}
			else if (Input.Close.get(0)<ATRTrailingStop.get(1) &&Input.Close.get(1)<ATRTrailingStop.get(1)){
				
				ATRTrailingStopVar  = Math.min(ATRTrailingStop.get(1), Input.Close.get(0)+Stop);
				}
			else if (Input.Close.get(0)>ATRTrailingStop.get(1)){

				ATRTrailingStopVar = Input.Close.get(0)-Stop;
				}
			else ATRTrailingStopVar = Input.Close.get(0)+Stop ;

			ATRTrailingStop.set(0,ATRTrailingStopVar);
		}
		else if(Method ==1){


			if (Input.High.get(0)>ATRTrailingStop.get(1) &&  Input.High.get(1)>ATRTrailingStop.get(1)){
								
				ATRTrailingStopVar  = Math.max(ATRTrailingStop.get(1), Input.Low.get(0)-Stop);

				}
			else if (Input.Low.get(0)<ATRTrailingStop.get(1) &&Input.Low.get(1)<ATRTrailingStop.get(1)){
				
				ATRTrailingStopVar  = Math.min(ATRTrailingStop.get(1), Input.High.get(0)+Stop);
				}
			else if (Input.High.get(0)>ATRTrailingStop.get(1)){

				ATRTrailingStopVar = Input.Low.get(0)-Stop;
				}
			else ATRTrailingStopVar = Input.High.get(0)+Stop ;

			ATRTrailingStop.set(0,ATRTrailingStopVar);

		}
		else if(Method ==2){

			if (Input.Close.get(0)>ATRTrailingStop.get(1) &&  Input.Close.get(1)>ATRTrailingStop.get(1)){
				
				ATRTrailingStopVar  = Math.max(ATRTrailingStop.get(1), Input.Low.get(0)-Stop);

				}
			else if (Input.Close.get(0)<ATRTrailingStop.get(1) &&Input.Close.get(1)<ATRTrailingStop.get(1)){
				
				ATRTrailingStopVar  = Math.min(ATRTrailingStop.get(1), Input.High.get(0)+Stop);
				}
			else if (Input.Close.get(0)>ATRTrailingStop.get(1)){

				ATRTrailingStopVar = Input.Low.get(0)-Stop;
				}
			else ATRTrailingStopVar = Input.High.get(0)+Stop ;

			ATRTrailingStop.set(0,ATRTrailingStopVar);

		}
				
	
		}

	}

	
}