package SQ.Blocks.Indicators.BHErgodic;

import com.strategyquant.lib.*;
import com.strategyquant.datalib.*;
import com.strategyquant.tradinglib.*;
import SQ.Calculators.AverageCalculator;

import SQ.Internal.IndicatorBlock;

/**
 * Indicator name as it will be displayed in UI, and its return type.
 * Possible return types:
 * ReturnTypes.Price - indicator is drawn on the price chart, like SMA, Bollinger Bands etc.
 * ReturnTypes.Number - indicator is drawn on separate chart, like CCI, RSI, MACD
 * ReturnTypes.PriceRange - indicator is price range, like ATR.
 */
@BuildingBlock(name="(BHE) BH Ergodic", display="BH Ergodic(@Chart@#R#,#S#,#U#,#Trigger#)[#Shift#]", returnType = ReturnTypes.Number)
@Indicator(oscillator=true, middleValue=0, min=-100, max=100, step=0.01)


public class BHErgodic extends IndicatorBlock {

	@Parameter(defaultChartIndex=0)
	public ChartData Chart;

	@Parameter(defaultValue="2", isPeriod=true, minValue=2, maxValue=1000, step=1)
	public int R;

	@Parameter(defaultValue="10", isPeriod=true, minValue=2, maxValue=1000, step=1)
	public int S;

	@Parameter(defaultValue="5", isPeriod=true, minValue=2, maxValue=1000, step=1)
	public int U;

	@Parameter(defaultValue="3", isPeriod=true, minValue=2, maxValue=1000, step=1)
	public int Trigger;

	@Output(name="TSI", color=Colors.Blue)
	public DataSeries TSI;


	@Output(name="Signal", color=Colors.Red)
	public DataSeries Signal;

	
	private AverageCalculator REmaAvgCalculator;
	private AverageCalculator SEmaAvgCalculator;
	private AverageCalculator UEmaAvgCalculator;

	private AverageCalculator aREmaAvgCalculator;
	private AverageCalculator aSEmaAvgCalculator;
	private AverageCalculator aUEmaAvgCalculator;
	private AverageCalculator finalEmaAvgCalculator;



	//------------------------------------------------------------------------
	//------------------------------------------------------------------------
	//------------------------------------------------------------------------

	protected void OnInit() throws TradingException {

		REmaAvgCalculator = new AverageCalculator(AverageCalculator.EMA, R);
		SEmaAvgCalculator = new AverageCalculator(AverageCalculator.EMA, S);
		UEmaAvgCalculator = new AverageCalculator(AverageCalculator.EMA, U);
		aREmaAvgCalculator = new AverageCalculator(AverageCalculator.EMA, R);
		aSEmaAvgCalculator = new AverageCalculator(AverageCalculator.EMA, S);
		aUEmaAvgCalculator = new AverageCalculator(AverageCalculator.EMA, U);
		finalEmaAvgCalculator = new AverageCalculator(AverageCalculator.EMA, Trigger);
	}


	@Override
	protected void OnBarUpdate() throws TradingException {
      double indiValue;

		if (CurrentBar < Math.max(R,Math.max(S,Math.max(U,Trigger)))) {

			TSI.set(0, 0);
			Signal.set(0, 0);


		} else {

			double priceDiff=Chart.Close.get(0)-Chart.Close.get(1);

			REmaAvgCalculator.onBarUpdate(priceDiff, CurrentBar);
			double rSM = REmaAvgCalculator.getValue();
			SEmaAvgCalculator.onBarUpdate(rSM, CurrentBar);
			double sSM = SEmaAvgCalculator.getValue();
			UEmaAvgCalculator.onBarUpdate(sSM, CurrentBar);
			double avg = UEmaAvgCalculator.getValue();

			aREmaAvgCalculator.onBarUpdate(Math.abs(priceDiff), CurrentBar);
			double arSM = aREmaAvgCalculator.getValue();
			aSEmaAvgCalculator.onBarUpdate(arSM, CurrentBar);
			double asSM = aSEmaAvgCalculator.getValue();
			aUEmaAvgCalculator.onBarUpdate(asSM, CurrentBar);
			double ava = aUEmaAvgCalculator.getValue();
			
			double tsi = (ava != 0) ? 100.0*avg/ava : 0;
			finalEmaAvgCalculator.onBarUpdate(tsi, CurrentBar);
						
			TSI.set(0, tsi);
			Signal.set(0, finalEmaAvgCalculator.getValue());

		}



	}



}