package SQ.Blocks.Indicators.ConnorsRSI;

import com.strategyquant.lib.*;
import com.strategyquant.datalib.*;
import com.strategyquant.tradinglib.*;

import SQ.Internal.IndicatorBlock;
import SQ.Calculators.*;
import SQ.Internal.IndicatorBlock;

/**
 * Indicator name as it will be displayed in UI, and its return type.
 * Possible return types:
 * ReturnTypes.Price - indicator is drawn on the price chart, like SMA, Bollinger Bands etc.
 * ReturnTypes.Number - indicator is drawn on separate chart, like CCI, RSI, MACD
 * ReturnTypes.PriceRange - indicator is price range, like ATR.
 */
@BuildingBlock(name="(CRSI) Connors RSI", display="Connors RSI(@Chart@#RSIPeriod#,#UpDownPeriod#)[#Shift#]", returnType = ReturnTypes.Number)
@Indicator(oscillator=true, middleValue=50, min=0, max=100, step=0.01)
public class CRSI extends IndicatorBlock {

	@Parameter(defaultChartIndex=0)
	public ChartData Chart;

	@Parameter(defaultValue="3", isPeriod=true, minValue=2, maxValue=1000, step=1)
	public int RSIPeriod;

	@Parameter(defaultValue="2", isPeriod=true, minValue=2, maxValue=1000, step=1)
	public int UpDownPeriod;


	@Output
	public DataSeries Value;

	@Buffer
	public DataSeries UpDown;

	@Buffer
	public DataSeries ROC;

	private RSICalculator trsiCalculator;
	private RSICalculator ursiCalculator;	



	//------------------------------------------------------------------------
	//------------------------------------------------------------------------
	//------------------------------------------------------------------------

	@Override
	protected void OnInit() throws TradingException {
		trsiCalculator = new RSICalculator(RSIPeriod);
		ursiCalculator = new RSICalculator(UpDownPeriod);

	}

	@Override
	protected void OnBarUpdate() throws TradingException {
      

		if (CurrentBar < Math.max(RSIPeriod,Math.max(UpDownPeriod,100))) {
			// what to set when there are less bars than indicator period parameter
			Value.set(0, 0);

		} else {
	
			if(Chart.Close.get(0)==Chart.Close.get(1)) UpDown.set(0,0);
			if(Chart.Close.get(0)<Chart.Close.get(1)) if(UpDown.get(1)>0) UpDown.set(0,-1); else UpDown.set(0,UpDown.get(1)-1);
			if(Chart.Close.get(0)>Chart.Close.get(1)) if(UpDown.get(1)<0) UpDown.set(0,1); else UpDown.set(0,UpDown.get(1)+1);
						 		
			trsiCalculator.onBarUpdate(Chart.Close.get(0), CurrentBar);
			ursiCalculator.onBarUpdate(UpDown.get(0), CurrentBar);
						
			if(Chart.Close.get(1)!=0) ROC.set(0,(Chart.Close.get(0)/Chart.Close.get(1)-1)*100);

			double ROC_Percent = 0;
			
			for (int k=1; k<=100; k++){

              	if (ROC.get(0) >= ROC.get(k)) ROC_Percent++;
			  	ROC_Percent = (ROC_Percent / 100) * 100;

			}
			double crsi = (trsiCalculator.getValue()+ursiCalculator.getValue()+ROC_Percent)/3;
			
			Value.set(0,crsi);
		
		}
	}
}