package SQ.Blocks.Indicators.DSSBressert;

import com.strategyquant.lib.*;
import com.strategyquant.datalib.*;
import com.strategyquant.tradinglib.*;
import SQ.Calculators.*;

import SQ.Blocks.Indicators.Stochastic.Stochastic;

import SQ.Internal.IndicatorBlock;

/**
 * Indicator name as it will be displayed in UI, and its return type.
 * Possible return types:
 * ReturnTypes.Price - indicator is drawn on the price chart, like SMA, Bollinger Bands etc.
 * ReturnTypes.Number - indicator is drawn on separate chart, like CCI, RSI, MACD
 * ReturnTypes.PriceRange - indicator is price range, like ATR.
 */
@BuildingBlock(name="(DSS) DSS Bressert", display="DSS Bressert(@Chart@#StochPeriod#,#EMAPeriod#,#TriggerPeriod#)[#Shift#]", returnType = ReturnTypes.Number)
@Indicator(oscillator=true, middleValue=50, min=0, max=100, step=0.01)
public class DSSBressert extends IndicatorBlock {

	@Parameter
	public ChartData Input;

	@Parameter(defaultValue="10", isPeriod=true, minValue=2, maxValue=1000, step=1)
	public int StochPeriod ;

	@Parameter(defaultValue="9", isPeriod=true, minValue=2, maxValue=1000, step=1)
	public int EMAPeriod;

	@Parameter(defaultValue="5", isPeriod=true, minValue=2, maxValue=1000, step=1)
	public int TriggerPeriod;

	@Buffer
	public DataSeries MIT;

	@Output(name="DSS", color=Colors.Red)
	public DataSeries DSS;

	@Output(name="Trigger", color=Colors.Blue)
	public DataSeries Trigger;

	private HighestCalculator highest;
	private LowestCalculator lowest;
	private HighestCalculator highRange;
	private LowestCalculator lowRange;
	private AverageCalculator triggerAvgCalc;

	//------------------------------------------------------------------------
	//------------------------------------------------------------------------
	//------------------------------------------------------------------------

	@Override
	protected void OnInit() throws TradingException {
		highest = new HighestCalculator(StochPeriod);
		lowest = new LowestCalculator(StochPeriod);
		highRange = new HighestCalculator(StochPeriod);
		lowRange = new LowestCalculator(StochPeriod);
		triggerAvgCalc = new AverageCalculator(AverageCalculator.EMA, TriggerPeriod);
	}

	@Override
	protected void OnBarUpdate() throws TradingException {

		double hg,lw,stoch;
		double tMIT, firstDelta, secondDelta;
		double smooth_coefficient = 2.0 / (1.0 + EMAPeriod);
		double tDSS;
		

		if (CurrentBar < Math.max(StochPeriod,Math.max(TriggerPeriod,EMAPeriod))) {

			MIT.set(0, 0);
			DSS.set(0, 0);
			Trigger.set(0, 0);

		} else {

			highest.onBarUpdate(Input.High.get(0), CurrentBar);
			lowest.onBarUpdate(Input.Low.get(0), CurrentBar);
			
			firstDelta = Input.Close.get(0) - lowest.getLowestValue();
			
			tMIT = firstDelta/(highest.getHighestValue() - lowest.getLowestValue())*100.0;
		
			MIT.set(0,smooth_coefficient * (tMIT - MIT.get(1)) + MIT.get(1)); 
	
			highRange.onBarUpdate(MIT.get(0), CurrentBar);
			lowRange.onBarUpdate(MIT.get(0), CurrentBar);

			secondDelta = MIT.get(0)- lowRange.getLowestValue();

			tDSS = secondDelta/(highRange.getHighestValue() - lowRange.getLowestValue())*100.0;
			
			DSS.set(0, smooth_coefficient * (tDSS - DSS.get(1)) + DSS.get(1));
			triggerAvgCalc.onBarUpdate(DSS.get(0), CurrentBar);
			Trigger.set(0,triggerAvgCalc.getValue());
		
		}
	}
}