package SQ.Blocks.Indicators.DonchianChannels;

import com.strategyquant.lib.*;
import com.strategyquant.datalib.*;
import com.strategyquant.tradinglib.*;
import SQ.Calculators.HighestCalculator;
import SQ.Calculators.LowestCalculator;

import SQ.Internal.IndicatorBlock;

/**
 * Indicator name as it will be displayed in UI, and its return type.
 * Possible return types:
 * ReturnTypes.Price - indicator is drawn on the price chart, like SMA, Bollinger Bands etc.
 * ReturnTypes.Number - indicator is drawn on separate chart, like CCI, RSI, MACD
 * ReturnTypes.PriceRange - indicator is price range, like ATR.
 */
@BuildingBlock(name="(DCH) Donchian Channels", display="Donchian Channels(#Period#).#Line#[#Shift#]", returnType = ReturnTypes.Price)
@Indicator(oscillator=false)
public class DonchianChannels extends IndicatorBlock {

	@Parameter
	public ChartData Chart;

	@Parameter(defaultValue="10", isPeriod=true, minValue=2, maxValue=1000, step=1)
	public int Period;

	@Output(name="Upper", color=Colors.Red)
	public DataSeries Upper;

	@Output(name="Middle", color=Colors.Blue)
	public DataSeries Middle;

	@Output(name="Lower", color=Colors.Red)
	public DataSeries Lower;


	private HighestCalculator upperCalculator;
	private LowestCalculator lowerCalculator;

	//------------------------------------------------------------------------
	//------------------------------------------------------------------------
	//------------------------------------------------------------------------

	@Override
	protected void OnInit() throws TradingException {

	upperCalculator = new HighestCalculator(Period);
	lowerCalculator = new LowestCalculator(Period);
	}


	@Override
	protected void OnBarUpdate() throws TradingException {
      double indiValue;

		if (CurrentBar < Period) {
			Upper.set(0,0);
			Middle.set(0,0);
			Lower.set(0,0);
			

		} else {

			upperCalculator.onBarUpdate(Chart.High.get(0), CurrentBar);
			lowerCalculator.onBarUpdate(Chart.Low.get(0), CurrentBar);

			Upper.set(0,upperCalculator.getHighestValue());
			Middle.set(0,(upperCalculator.getHighestValue()+lowerCalculator.getLowestValue())/2);
			Lower.set(0,lowerCalculator.getLowestValue());
			
		}

	}
}