package SQ.Blocks.Indicators.HullMovingAverageATRBands;

import com.strategyquant.lib.*;
import com.strategyquant.datalib.*;
import com.strategyquant.tradinglib.*;
import SQ.Blocks.Indicators.MTATR.MTATR;
import SQ.Blocks.Indicators.ATR.ATR;
import SQ.Blocks.Indicators.HullMovingAverage.HullMovingAverage;


import SQ.Internal.IndicatorBlock;

/**
 * Indicator name as it will be displayed in UI, and its return type.
 * Possible return types:
 * ReturnTypes.Price - indicator is drawn on the price chart, like SMA, Bollinger Bands etc.
 * ReturnTypes.Number - indicator is drawn on separate chart, like CCI, RSI, MACD
 * ReturnTypes.PriceRange - indicator is price range, like ATR.
 */
@BuildingBlock(name="(HMAATRB) Hull Moving Average ATR Bands", display="Hull Moving Average ATR Bands (@Chart@#Period#,#Multiplication#,#ComputedFrom#).#Line#[#Shift#]", returnType = ReturnTypes.Price)
@Help("")
public class HullMovingAverageATRBands extends IndicatorBlock {

	@Parameter
	public ChartData Chart;

	@Parameter(defaultValue="0")
	@Editor(type=Editors.Selection, values="Close=0,Open=1,High=2,Low=3,Median=4,Typical=5,Weighted=6")
	public int ComputedFrom;

	@Parameter(defaultValue="24", isPeriod=true, minValue=2, maxValue=1000, step=1)
	public int Period;

	@Parameter(defaultValue="1.5", isPeriod=false, minValue=0.1, maxValue=10, step=0.1)
	public double Multiplication;


    @Output(name = "Upper", color = Colors.Blue)
    public DataSeries Upper;

    @Output(name = "Lower", color = Colors.Blue)
    public DataSeries Lower;

	//------------------------------------------------------------------------
	//------------------------------------------------------------------------
	//------------------------------------------------------------------------

	/**
	 * This method is called on every bar update and here the indicator value is computed.
	 *
	 * Unlike in MT4 you don't compute indicator values for multiple bars in a loop, 
	 * you need to compute value only for the latest (current) bar.
	 * Trading engine will take care of calling this method for every bar in the chart.
	 *
	 * Actual bar for which the indicator value is computed is stored in CurrentBar variable. 
	 * If 0, it means it is the very first bar of the chart.
	 */
	@Override
	protected void OnBarUpdate() throws TradingException {
      double indiValue;

		if (getCurrentBar() < Period) {
			// what to set when there are less bars than indicator period parameter
			Upper.set(0, 0);
			Lower.set(0, 0);
			//HMAMiddle.set(0, 0);

		} else {


			HullMovingAverage HMAindicator = Indicators.HullMovingAverage(Chart.getSeries(ComputedFrom), Period);
  	  		double hma = HMAindicator.Value.get(0);

			MTATR ATRindicator = Indicators.MTATR(Chart, Period);
  	  		double atrValue = ATRindicator.Value.get(0);

			Upper.set(0, hma+ Multiplication*atrValue);
			Lower.set(0, hma- Multiplication*atrValue);
		}

      // set indicator value to a Value output array.
      // The array is zero based, index 0 means it is value at current bar.
     
	}
}