package SQ.Blocks.Indicators.HullMovingAverage;

import com.strategyquant.datalib.*;
import com.strategyquant.tradinglib.*;
import SQ.Calculators.*;
import SQ.Internal.IndicatorBlock;
import java.lang.*;


@BuildingBlock(name="(HMABB) Hull Moving Average Bollinger Bands", display="Hull Moving Average Bollinger Bands(@Chart@#Period#,#Deviation#,#ComputedFrom#).#Line#[#Shift#]", returnType = ReturnTypes.Price)
@Help("")

public class HullMovingAverageBollingerBands extends IndicatorBlock {

    @Parameter(defaultValue="Close")
    public DataSeries Input;

    @Parameter(defaultValue="24", isPeriod=true, minValue=2, maxValue=1000, step=1)
    public int Period;

	@Parameter(defaultValue="2", isPeriod=false, minValue=0.1, maxValue=10, step=0.1)
	public double Deviation;

    @Output(name = "Upper", color = Colors.Blue)
    public DataSeries Upper;

    @Output(name = "Lower", color = Colors.Blue)
    public DataSeries Lower;

    private AverageCalculator fastwma;
    private AverageCalculator slowwma;
    private AverageCalculator hma;
    
	private int fastMaPeriod;
	private int squaredPeriod;
    private StdDevCalculator stdDevCalculator;

    //------------------------------------------------------------------------
    //------------------------------------------------------------------------
    //------------------------------------------------------------------------

    @Override
    protected void OnInit() throws TradingException {
    
	fastMaPeriod = (int)Math.floor(Period/2);
	squaredPeriod =  (int)Math.floor(Math.sqrt(Period));
   
    fastwma =  new AverageCalculator(AverageCalculator.LWMA, fastMaPeriod);
    slowwma = new AverageCalculator(AverageCalculator.LWMA, Period);
    hma = new AverageCalculator(AverageCalculator.LWMA, squaredPeriod);
    stdDevCalculator = new StdDevCalculator(Period);

    /*    @Output(name = "Middle", color = Colors.Red)
    public DataSeries Middle;*/
    
    }

    @Override
    protected void OnBarUpdate() throws TradingException {

        if(getCurrentBar()>Period){
        double hmainner;
       
        int fastMaPeriodT = (int)Math.floor(Period/2);
	    int squaredPeriodT =  (int)Math.floor(Math.sqrt(Period));
        
        fastwma.onBarUpdate(Input.get(0), getCurrentBar());
        slowwma.onBarUpdate(Input.get(0), getCurrentBar());
        
        double fma = fastwma.getValue();
        double sma = slowwma.getValue();
       
        hmainner =  ((2* fma) - sma);
        hma.onBarUpdate(hmainner, getCurrentBar());
     
        //Middle.set(0, hma.getValue());

        stdDevCalculator.onBarUpdate(hma.getValue(), getCurrentBar());
        
        double stdDevValue = stdDevCalculator.getValue();

        Upper.set(0, hma.getValue() + Deviation * stdDevValue);
        Lower.set(0, hma.getValue() - Deviation * stdDevValue);
        }
        else return;
 
    }
}

