package SQ.Blocks.Indicators.LogATR;

import com.strategyquant.lib.*;
import com.strategyquant.datalib.*;
import com.strategyquant.tradinglib.*;

import SQ.Internal.IndicatorBlock;

/**
 * Indicator name as it will be displayed in UI, and its return type.
 * Possible return types:
 * ReturnTypes.Price - indicator is drawn on the price chart, like SMA, Bollinger Bands etc.
 * ReturnTypes.Number - indicator is drawn on separate chart, like CCI, RSI, MACD
 * ReturnTypes.PriceRange - indicator is price range, like ATR.
 */
@BuildingBlock(name="(LATR) LogATR", display="LogATR(@Chart@#Period#)[#Shift#]", returnType = ReturnTypes.Number)
@Help("Average True Range of log prices")
public class LogATR extends IndicatorBlock {

	@Parameter(defaultChartIndex=0)
	public ChartData Chart;

	@Parameter(defaultValue="10", isPeriod=true, minValue=2, maxValue=1000, step=1)
	public int Period;

	@Output
	public DataSeries Value;

	//------------------------------------------------------------------------
	//------------------------------------------------------------------------
	//------------------------------------------------------------------------

	@Override
	protected void OnBarUpdate() throws TradingException {
		if (CurrentBar == 0){
			Value.set(0, Math.log(Chart.High.get(0)) - Math.log(Chart.Low.get(0)));
		} else {
			double trueRangeSum = 0.0;
			int barsAgoMax = Math.min(Period, CurrentBar);
			
			for(int barsAgo = 0; barsAgo < barsAgoMax; barsAgo++) {
				double high = Math.log(Chart.High.get(barsAgo));
				double low = Math.log(Chart.Low.get(barsAgo));
				double prevClose = Math.log(Chart.Close.get(barsAgo + 1));
					
				double barRange = high - low;
				double lowCloseRange = Math.abs(low - prevClose);
				double highCloseRange = Math.abs(high - prevClose);
				double trueRange = Math.max(barRange, Math.max(lowCloseRange, highCloseRange));
				
				trueRangeSum += trueRange;
			}

			double averageTrueRange = trueRangeSum / barsAgoMax;
			
			Value.set(0, averageTrueRange);
		}
	}
}