package SQ.Blocks.Indicators.MAMAFAMAKAMA;

import com.strategyquant.lib.*;
import com.strategyquant.datalib.*;
import com.strategyquant.tradinglib.*;
import SQ.Calculators.SumCalculator;

import SQ.Internal.IndicatorBlock;


@BuildingBlock(name="(MFK) MAMAFAMAKAMA", display="MAMAFAMAKAMA(@Chart@#Period#).#Line#[#Shift#]", returnType = ReturnTypes.Price)
public class MAMAFAMAKAMA extends IndicatorBlock {

	@Parameter(name="Input", defaultValue="Close")
	public DataSeries Input;

	@Parameter(defaultValue="10", isPeriod=true, minValue=2, maxValue=1000, step=1)
	public int Period;

	@Output(name="MAMA", color=Colors.Blue)
	public DataSeries MAMA;

	@Output(name="FAMA", color=Colors.Red)
	public DataSeries FAMA;

	@Output(name="KAMA", color=Colors.Green)
	public DataSeries KAMA;

	@Buffer
	public DataSeries mesaPeriod;
	@Buffer
	public DataSeries I1;
	@Buffer
	public DataSeries Q1;
	@Buffer
	public DataSeries jI;
	@Buffer
	public DataSeries jQ;
	@Buffer
	public DataSeries detrender;
	@Buffer
	public DataSeries smooth;
	@Buffer
	public DataSeries I2;
	@Buffer
	public DataSeries Q2;
	@Buffer
	public DataSeries Re;
	@Buffer
	public DataSeries Im;
	@Buffer
	public DataSeries phase;
	
	private SumCalculator sumCalc;


	//------------------------------------------------------------------------
	//------------------------------------------------------------------------
	//------------------------------------------------------------------------

	@Override
	protected void OnInit() throws TradingException {
		sumCalc = new SumCalculator(Period);
		
	}
	@Override
	protected void OnBarUpdate() throws TradingException {


		if (CurrentBar < Period) {
			
			MAMA.set(0,0);
			FAMA.set(0,0);
			KAMA.set(0,0);
			I1.set(0,0);
			Q1.set(0,0);
			jI.set(0,0);
			jQ.set(0,0);
			I2.set(0,0);
			Q2.set(0,0);
			Re.set(0,0);
			Im.set(0,0);
			mesaPeriod.set(0,0);
			phase.set(0,0);
			detrender.set(0,0);
			smooth.set(0,0);

		} else {


			double mesaPeriodMult = 0.075 * mesaPeriod.get(1) + 0.54;			
			smooth.set(0,(4 * Input.get(0) + 3 * Input.get(1) + 2 * Input.get(2) + Input.get(3)) / 10);
						
			double det = hilbertTransform(smooth)*mesaPeriodMult;
			detrender.set(0,det);
					
			I1.set(0,detrender.get(3));
			Q1.set(0,hilbertTransform(detrender)*mesaPeriodMult);
			
			jI.set(0,hilbertTransform(I1)*mesaPeriodMult);
			jQ.set(0,hilbertTransform(Q1)*mesaPeriodMult);
			
			double I2Var = I1.get(0)-jQ.get(0);
			double Q2Var = Q1.get(0)+jI.get(0);

			I2.set(0,0.2 * I2Var + 0.8 * I2.get(1));
			Q2.set(0,0.2 * Q2Var + 0.8 * Q2.get(1));
			
			double ReVar = I2Var * I2.get(1) + Q2Var * Q2.get(1);
			double ImVar = I2Var * Q2.get(1) - Q2Var * I2.get(1);
			Re.set(0,SQUtils.round6(0.2 * ReVar + 0.8 *Re.get(1)));
			Im.set(0,SQUtils.round6(0.2 * ImVar + 0.8 *Im.get(1)));
			//MAMA.set(0,Im.get(0));	

			double  mesaPeriodVar = 0;
			if(Re.get(0)!=0 && Im.get(0)!=0) mesaPeriodVar = (2 * Math.PI)  / Math.atan(SQUtils.safeDivide(Im.get(0), Re.get(0))) ;	
			if(mesaPeriodVar>1.5 * mesaPeriod.get(1)) mesaPeriodVar = 1.5 * mesaPeriod.get(1);
			if(mesaPeriodVar<0.67 * mesaPeriod.get(1)) mesaPeriodVar = 0.67 * mesaPeriod.get(1);
			if(mesaPeriodVar<6) mesaPeriodVar = 6;
			if(mesaPeriodVar>50) mesaPeriodVar = 50;
			
			mesaPeriod.set(0, SQUtils.round(0.2 * mesaPeriodVar + 0.8 * mesaPeriod.get(1),4));
			
			double phaseVar = 0;
			if (I1.get(0) !=0) phase.set(0, SQUtils.round(SQUtils.safeDivide((180/Math.PI), (Math.atan(Q1.get(0) / I1.get(0)) )), 2));
			double deltaPhasaVar = phase.get(1)- phase.get(0);
			
			if (deltaPhasaVar<1) deltaPhasaVar = 1;
			sumCalc.onBarUpdate(Math.abs((Input.get(0)-Input.get(1))), getCurrentBar());
			
			double sumCalcVar = sumCalc.getValue();
			double er = SQUtils.safeDivide((Input.get(0)-Input.get(Period)),sumCalcVar);
			double  alpha =  SQUtils.safeDivide(er, deltaPhasaVar);		
			
			if(alpha<(er*.01)) alpha = er*.01;
			
			double a  = alpha;
			double b   = alpha/2.0d;
			
			MAMA.set(0,a*Input.get(0)+(1-a)*MAMA.get(1)); 
			FAMA.set(0,b*MAMA.get(0)+(1-b)*FAMA.get(1)); 
			
			alpha = Math.pow((er*(b-a))+a, 2); 
			
			double kama  = SQUtils.round((alpha*Input.get(0))+(1-alpha)*KAMA.get(1),5);
			KAMA.set(0,kama);
			

		}  
	}


private double hilbertTransform(DataSeries Input)throws TradingException {

	double val = 0.0962 * Input.get(0) + 0.5769 * Input.get(2) - 0.5769 * Input.get(4) - 0.0962 * Input.get(6);

	return val;
}


}
