package SQ.Blocks.Indicators.MoneyFlowIndex;

import com.strategyquant.lib.*;
import com.strategyquant.datalib.*;
import com.strategyquant.tradinglib.*;

import SQ.Internal.IndicatorBlock;

@BuildingBlock(name="(MFI) Money Flow Index", display="Money Flow Index(@Chart@#Period#)[#Shift#]", returnType = ReturnTypes.Number)
@Indicator(oscillator=true, middleValue=50, min=0, max=100, step=0.01)
@ParameterSet(set="Period=14")
@ParameterSet(set="Period=20")
@ParameterSet(set="Period=30")
@ParameterSet(set="Period=40")
@ParameterSet(set="Period=50")
@ParameterSet(set="Period=100")
@ParameterSet(set="Period=12")
@ParameterSet(set="Period=24")
@ParameterSet(set="Period=48")
@ParameterSet(set="Period=96")
@ParameterSet(set="Period=120")
public class MoneyFlowIndex extends IndicatorBlock {

	@Parameter(defaultChartIndex=0)
	public ChartData Chart;

	@Parameter(defaultValue="14", isPeriod=true, minValue=2, maxValue=1000, step=1)
	public int Period;

	@Output(name = "MFI", color = Colors.Red)
	public DataSeries Value;

	//------------------------------------------------------------------------
	//------------------------------------------------------------------------
	//------------------------------------------------------------------------

	@Override
	protected void OnBarUpdate() throws TradingException {

		double indiValue;
		double dCurrentTP = 0;
		double dPreviousTP = 0;
		double posMFI = 0;
		double negMFI = 0;

		if (CurrentBar < Period) {

			Value.set(0, 0);

		} else {
			posMFI = 0;
			negMFI = 0;
			double typPrice = Chart.Typical.get(0);
			double volume = Chart.Volume.get(0);
			double rawMFI = volume*typPrice;

			for( int i = 0; i<Period;i++){

				dCurrentTP =  Chart.Typical.get(i);
				dPreviousTP = Chart.Typical.get(i+1);

				if(dCurrentTP>dPreviousTP)  posMFI = posMFI+Chart.Typical.get(i)*Chart.Volume.get(i);
				else 
				{
					if(dCurrentTP<dPreviousTP) negMFI = negMFI+Chart.Typical.get(i)*Chart.Volume.get(i);
				}
				dCurrentTP=dPreviousTP;  
			}

			double ratioMFI = posMFI/negMFI;
			double MFI = 100-(100/(1+ratioMFI));

			//Value.set(0, MFI);
			Value.set(0, MFI);
		}
	}
}