package SQ.Blocks.Indicators.RVI;

import com.strategyquant.lib.*;
import com.strategyquant.datalib.*;
import com.strategyquant.tradinglib.*;

import SQ.Internal.IndicatorBlock;

/**
 * Indicator name as it will be displayed in UI, and its return type.
 * Possible return types:
 * ReturnTypes.Price - indicator is drawn on the price chart, like SMA, Bollinger Bands etc.
 * ReturnTypes.Number - indicator is drawn on separate chart, like CCI, RSI, MACD
 * ReturnTypes.PriceRange - indicator is price range, like ATR.
 */
@BuildingBlock(name="(RVI) RVI", display="RVI(@Chart@#Period#)[#Shift#]", returnType = ReturnTypes.Number)
@Indicator(oscillator=true, middleValue=0, min=-1, max=1, step=0.01)
@Help("RVI help text")
public class RVI extends IndicatorBlock {

	@Parameter(defaultChartIndex=0)
	public ChartData Chart;

	@Parameter(defaultValue="10", isPeriod=true, minValue=2, maxValue=1000, step=1)
	public int Period;

	@Output
	public DataSeries RVI;

	@Output
	public DataSeries RVISignal;

	//------------------------------------------------------------------------
	//------------------------------------------------------------------------
	//------------------------------------------------------------------------

	@Override
	protected void OnBarUpdate() throws TradingException {
      double indiValue;
	  double dValueUp,dValueDown;

		if (CurrentBar < Period) {
			// what to set when there are less bars than indicator period parameter
			RVI.set(0, 0);
			RVISignal.set(0, 0);

		} else {
			double dNum=0;
			double dDeNum=0;

			for(int j = 0;j<Period;j++){
				
				dValueUp=((Chart.Close.get(j)-Chart.Open.get(j))+2*(Chart.Close.get(j+1)-Chart.Open.get(j+1))+2*(Chart.Close.get(j+2)-Chart.Open.get(j+2))+(Chart.Close.get(j+3)-Chart.Open.get(j+3)))/6;
				dValueDown=((Chart.High.get(j)-Chart.Low.get(j))+2*(Chart.High.get(j+1)-Chart.Low.get(j+1))+2*(Chart.High.get(j+2)-Chart.Low.get(j+2))+(Chart.High.get(j+3)-Chart.Low.get(j+3)))/6;
				dNum = dNum + dValueUp;
				dDeNum=dDeNum+dValueDown;

			}
			
			RVI.set(0, SQUtils.safeDivide(dNum, dDeNum));
		
			double rviSig = (RVI.get(0)+2*RVI.get(1)+2*RVI.get(2)+RVI.get(3))/6;
			RVISignal.set(0,rviSig);
		}

     
	}
}