package SQ.Blocks.Indicators.SlopeDirectionLine;

import com.strategyquant.lib.*;
import com.strategyquant.datalib.*;
import com.strategyquant.tradinglib.*;
import SQ.Calculators.AverageCalculator;
import com.strategyquant.tradinglib.Colors;
import com.strategyquant.tradinglib.Editor;
import com.strategyquant.tradinglib.Editors;
import com.strategyquant.tradinglib.Help;
import com.strategyquant.tradinglib.IgnoreInBuilder;
import com.strategyquant.tradinglib.Output;
import com.strategyquant.tradinglib.Parameter;
import com.strategyquant.tradinglib.ParameterSet;
import com.strategyquant.tradinglib.ReturnTypes;
import SQ.Internal.IndicatorBlock;
import SQ.Calculators.AverageCalculator;

/**
 * Indicator name as it will be displayed in UI, and its return type.
 * Possible return types:
 * ReturnTypes.Price - indicator is drawn on the price chart, like SMA, Bollinger Bands etc.
 * ReturnTypes.Number - indicator is drawn on separate chart, like CCI, RSI, MACD
 * ReturnTypes.PriceRange - indicator is price range, like ATR.
 */
@BuildingBlock(name="(SDL) Slope Direction Line", display="Slope Direction Line(@Chart@#ComputedFrom#,#MAPeriod#,#MAMethod#)[#Shift#]", returnType = ReturnTypes.Price)
@Indicator(oscillator=false)
public class SlopeDirectionLine extends IndicatorBlock {

	@Parameter(name="Input", defaultValue="0")
	public DataSeries Input;

	@Parameter(defaultValue="80", isPeriod=true, minValue=2, maxValue=1000, step=1)
	public int MAPeriod;
	
	@Parameter(name="Method", defaultValue="0")
	@Editor(type=Editors.Selection, values="Simple=0,Exponential=1,Smoothed=2,Linear weighted=3")
	public int MAMethod;

	@Output
	public DataSeries Value;


	private AverageCalculator averageCalculator1;
	private AverageCalculator averageCalculator2;
	private AverageCalculator averageCalculator3;

	//------------------------------------------------------------------------
	//------------------------------------------------------------------------
	//------------------------------------------------------------------------

	@Override
	protected void OnInit() throws TradingException {

		int new_period = (int)Math.max((MAPeriod/2),1);
		int new_period_sqrt=(int)Math.max(Math.sqrt(MAPeriod),1);

		MAMethod = SQUtils.fixAllowedRange(MAMethod, 0, 3, 0);

		averageCalculator1 = new AverageCalculator(MAMethod, Math.max(MAPeriod,2));
		averageCalculator2 = new AverageCalculator(MAMethod, new_period);
		averageCalculator3 = new AverageCalculator(MAMethod, new_period_sqrt);
	}

	@Override
	protected void OnBarUpdate() throws TradingException {
     
		if (CurrentBar < MAPeriod) {
			// what to set when there are less bars than indicator period parameter
			Value.set(0, 0);

		} else {

			averageCalculator1.onBarUpdate(Input.get(0), CurrentBar);
			averageCalculator2.onBarUpdate(Input.get(0), CurrentBar);

			double ma1 = averageCalculator1.getValue();
			double ma2 = averageCalculator2.getValue();
		
			double vect = 2*ma2-ma1;
			averageCalculator3.onBarUpdate(vect, CurrentBar);
			double ma3 = averageCalculator3.getValue();

			Value.set(0, ma3);
		}
	}
}