package SQ.Blocks.Indicators.SmoothedRSI;

import com.strategyquant.lib.*;
import com.strategyquant.datalib.*;
import com.strategyquant.tradinglib.*;
import com.strategyquant.datalib.DataSeries;
import com.strategyquant.datalib.TradingException;
import com.strategyquant.tradinglib.BuildingBlock;
import com.strategyquant.tradinglib.Colors;
import com.strategyquant.tradinglib.Indicator;
import com.strategyquant.tradinglib.Output;
import com.strategyquant.tradinglib.Parameter;
import com.strategyquant.tradinglib.ParameterSet;
import com.strategyquant.tradinglib.ReturnTypes;
import SQ.Calculators.AverageCalculator;

import SQ.Calculators.RSICalculator;
import SQ.Internal.IndicatorBlock;

import SQ.Internal.IndicatorBlock;


@BuildingBlock(name="(SRSI) Smoothed RSI", display="Smoothed RSI(@Chart@#RSIPeriod#,#MAPeriod#,#MAMethod#).#Line#[#Shift#]", returnType = ReturnTypes.Number)
@Indicator(oscillator=true, middleValue=50, min=0, max=100, step=0.01)
@Help("SmoothedRSI help text")
public class SmoothedRSI extends IndicatorBlock {

	@Parameter(name="Input", defaultValue="0")
	public DataSeries Input;
	
	@Parameter(defaultValue="14", minValue=2, maxValue=10000, step=1)
	public int RSIPeriod;

	@Parameter(name="Method", defaultValue="0")
	@Editor(type=Editors.Selection, values="Simple=0,Exponential=1,Smoothed=2,Linear weighted=3")
	public int MAMethod;

	@Parameter(defaultValue="5")
	public int MAPeriod;

	@Output(name = "RSI", color = Colors.Red)
	public DataSeries RSI;

	@Output(name = "MA", color = Colors.Blue)
	public DataSeries MARSI;


	private RSICalculator rsiCalculator;
	private AverageCalculator averageCalculator;
	
	//------------------------------------------------------------------------
	//------------------------------------------------------------------------
	//------------------------------------------------------------------------

	@Override
	protected void OnInit() throws TradingException {
		rsiCalculator = new RSICalculator(RSIPeriod);
		MAMethod = SQUtils.fixAllowedRange(MAMethod, 0, 3, 0);
		averageCalculator = new AverageCalculator(MAMethod, MAPeriod);
	}
	
	//------------------------------------------------------------------------
	
	@Override
	protected void OnBarUpdate() throws TradingException {

		rsiCalculator.onBarUpdate(Input.get(0), CurrentBar);
		double rsi = rsiCalculator.getValue();
		averageCalculator.onBarUpdate(rsi, CurrentBar);
		double maRSI = averageCalculator.getValue();
		
		RSI.set(0, rsi);
		MARSI.set(0, maRSI);

	}

}