package SQ.Blocks.Indicators.TTMSqueeze;

import com.strategyquant.lib.*;
import com.strategyquant.datalib.*;
import com.strategyquant.tradinglib.*;
import SQ.Blocks.Indicators.ATR.ATR;
import SQ.Blocks.Indicators.BollingerBands.BollingerBands;
import SQ.Blocks.Indicators.KeltnerChannel.KeltnerChannel;
import SQ.Blocks.Indicators.Momentum.Momentum;
import SQ.Blocks.Indicators.LinReg.LinearRegression;
import SQ.Internal.IndicatorBlock;


@BuildingBlock(name="(TTMS) TTM Squeeze", display="TTM Squeeze(@Chart@#BBPeriod#,#KCPeriod#,#LinRegPeriod#,#MomPeriod#,#BBDeviation#,#NumATRs#).#Line#[#Shift#]", returnType = ReturnTypes.Number)
@Help("TTM Squeeze help text")
@Indicator(oscillator=true, middleValue=0, min=-1000, max=1000, step=0.001)
public class TTMSqueeze extends IndicatorBlock {

	@Parameter
	public ChartData Chart;

	@Parameter(defaultValue="20", isPeriod=true, minValue=2, maxValue=1000, step=1)
	public int BBPeriod;

	@Parameter(defaultValue="20", isPeriod=true, minValue=2, maxValue=1000, step=1)
	public int KCPeriod;

	@Parameter(defaultValue="20", isPeriod=true, minValue=2, maxValue=1000, step=1)
	public int LinRegPeriod;

	@Parameter(defaultValue="20", isPeriod=true, minValue=2, maxValue=1000, step=1)
	public int MomPeriod;

	@Parameter(defaultValue="2", isPeriod=true, minValue=0.1, maxValue=10, step=0.1)
	public double BBDeviation;

	@Parameter(defaultValue="2", isPeriod=true, minValue=0.1, maxValue=10, step=0.1)
	public double NumATRs;


	@Output(name="Squeeze", color=Colors.Green)
	public DataSeries Squeeze;
	@Output(name="Squeeze Momentum", color=Colors.Red)
	public DataSeries SqueezeMomentum;

	//------------------------------------------------------------------------
	//------------------------------------------------------------------------
	//------------------------------------------------------------------------


	@Override
	protected void OnBarUpdate() throws TradingException {
		boolean squeeze = false;

		
	
		if (CurrentBar < Math.max(Math.max(Math.max(BBPeriod, KCPeriod),LinRegPeriod),MomPeriod)) {

			SqueezeMomentum.set(0, 0);
			Squeeze.set(0, 0);

		}
		else{

			BollingerBands BBIndicator = Indicators.BollingerBands(Chart.Close, BBPeriod,BBDeviation);
			double bbUpper = BBIndicator.Upper.getRounded(Shift);
			double bblower = BBIndicator.Lower.getRounded(Shift);

			KeltnerChannel​ KCIndicator = Indicators.KeltnerChannel(Chart, KCPeriod,NumATRs);
			double kcUpper = KCIndicator.Upper.getRounded(Shift);
			double kcLower = KCIndicator.Lower.getRounded(Shift);

			LinearRegression LRIndicator = Indicators.LinearRegression(Chart.Close, LinRegPeriod);
			double lrValue = LRIndicator.Value.getRounded(Shift);
			double lrMomValue = LRIndicator.Value.getRounded(Shift+MomPeriod);
	
			//	Upper Bollinger Band <= Upper Keltner Channel
			//	Lower Bollinger Band >= Lower Keltner Channel
			
			if(bbUpper<=kcUpper&bblower>=kcLower) {
				squeeze = true;
				Squeeze.set(1);
				}
				else Squeeze.set(0);

			SqueezeMomentum.set(LRMomentum(LRIndicator,MomPeriod));
	
		}
	
	}


private double LRMomentum(LinearRegression LRIndicator , int Period) throws TradingException{

	/// SQ_Momemtum
	//double rv = LRIndicator.Value.getRounded(Shift)*100.0/ LRIndicator.Value.getRounded(Shift+Period);
	///TS_Momemtum
	double rv = LRIndicator.Value.getRounded(Shift)- LRIndicator.Value.getRounded(Shift+Period);

	return rv;
	}


}