package SQ.Blocks.Indicators.ZScore;

import com.strategyquant.lib.*;
import com.strategyquant.datalib.*;
import com.strategyquant.tradinglib.*;
import SQ.Calculators.*;

import SQ.Internal.IndicatorBlock;

/**
 * Indicator name as it will be displayed in UI, and its return type.
 * Possible return types:
 * ReturnTypes.Price - indicator is drawn on the price chart, like SMA, Bollinger Bands etc.
 * ReturnTypes.Price - indicator is drawn on separate chart, like CCI, RSI, MACD
 * ReturnTypes.PriceRange - indicator is price range, like ATR.
 */
@BuildingBlock(name="(ZS) ZScore", display="ZScore(@Chart@#ComputedFrom#,#Period#)[#Shift#]", returnType = ReturnTypes.Number)
@Indicator(oscillator=true, middleValue=0, min=-4, max=4, step=0.0001)
@Help("ZScore help text")
public class ZScore extends IndicatorBlock {

    
	@Parameter(name="Input", defaultValue="0")
	public DataSeries Input;
    
    @Parameter(defaultValue="20", isPeriod=true, minValue=2, maxValue=1000, step=1)
    public int Period;

    @Output
    public DataSeries Value;

    private AverageCalculator avgCalculator;
    private StdDevCalculator stdevCalculator;
	

    //------------------------------------------------------------------------
    //------------------------------------------------------------------------
    //------------------------------------------------------------------------

    /**
     * This method is called on every bar update and here the indicator value is computed.
     *
     * Unlike in MT4 you don't compute indicator values for multiple bars in a loop, 
     * you need to compute value only for the latest (current) bar.
     * Trading engine will take care of calling this method for every bar in the chart.
     *
     * Actual bar for which the indicator value is computed is stored in CurrentBar variable. 
     * If 0, it means it is the very first bar of the chart.
     */

    
        @Override
    protected void OnInit() throws TradingException {

    avgCalculator = new AverageCalculator(AverageCalculator.SMA,Period);
    stdevCalculator = new StdDevCalculator(Period);
        
    }


    @Override
    protected void OnBarUpdate() throws TradingException {

        if (CurrentBar < Period) {
			// what to set when there are less bars than indicator period parameter
			Value.set(0,0);

		} 
        else {
      
            stdevCalculator.onBarUpdate(Input.get(0), CurrentBar); //// Set input (Close) to stdev calc
            avgCalculator.onBarUpdate(Input.get(0), CurrentBar); //// Set input (Close) to avg calc
            
            double sma = avgCalculator.getValue(); /// get value from calculator
            double stdev = stdevCalculator.getValue(); /// get value from calculator
            
            Value.set(0, (Input.get(0)-sma)/stdev);
        }
    }
}

