package SQ.Columns.Databanks;

import com.strategyquant.lib.*;
import com.strategyquant.datalib.*;
import com.strategyquant.tradinglib.*;

public class AvgExitQuality extends DatabankColumn {
    
	public AvgExitQuality() {
		super(L.t("Avg. Exit Quality"), DatabankColumn.Decimal2, ValueTypes.Maximize, 0, 0, 50);
		
		setDependencies("NumberOfTrades");
		setTooltip(L.t("Avg. Exit Quality"));
	}
	
	//------------------------------------------------------------------------

	@Override
	public double compute(SQStats stats, StatsTypeCombination combination, OrdersList ordersList, SettingsMap settings, SQStats statsLong, SQStats statsShort) throws Exception {
        

		//// http://www.automated-trading-system.com/e-ratio-trading-edge/ as proposed here
		double normMAE = 0;
		double normMFE = 0;
		double exitQuality =0;
		int count = 0;

		for(int i = 0; i<ordersList.size(); i++) {
			Order order = ordersList.get(i);
			
			if(order.isBalanceOrder() || !order.isRealOrder()) {
				// don't count balance orders (deposits, withdrawals) in
				continue;
			}

			if(order.PipsPL<=0 ) {
				// don't count negative PL  orders  in
				continue;
			}
			
			//normMAE += (order.PipsMAE/order.ATROnOpen);
			//normMFE += (order.PipsMFE/order.ATROnOpen);
			exitQuality += order.PipsPL/order.PipsMFE;
			count++;
			
		}

		//int numberOfTrades = stats.getInt("NumberOfTrades");

		//double avgNormMAE = normMAE/numberOfTrades;
		//double avgNormMFE = normMFE/numberOfTrades;

		return round2(safeDivide(exitQuality, count));	 /// edge ratio	
	}
	
}