package SQ.Columns.Databanks;

import com.strategyquant.lib.*;
import com.strategyquant.datalib.*;
import com.strategyquant.tradinglib.*;
import com.strategyquant.tradinglib.results.SpecialValues;
import com.strategyquant.tradinglib.ResultTypes;

public class AvgHoursInTrade extends DatabankColumn {
    
	public AvgHoursInTrade() {
		super(L.tsq("Avg. Hours in Trade"), DatabankColumn.Decimal4, ValueTypes.Minimize, 0, 0, 40);
		
		setTooltip(L.tsq("Average Hours In Trade"));		
	}

	//------------------------------------------------------------------------

	@Override
	public double compute(SQStats stats, StatsTypeCombination combination, OrdersList ordersList, SettingsMap settings, SQStats statsLong, SQStats statsShort, Result result, SettingsMap rgSpecialValues) throws Exception {
		int hoursTrades = 0;
		double multi = 0;

		if(result == null){
				return 0;

		}

		ResultsGroup rg =  result.getResultsGroup();
        if(rg == null) {
            return 0;
        }
       
        Result res = rg.subResult(rg.getMainResultKey());
        if(res == null) {
            return 0;
        }

        
		String tf = res.getString(SpecialValues.Timeframe);
        if(tf == null) {
            return 0;
        }

		switch(tf){
			case "H1":	multi = 1;
			break;
			case "H2":	multi = 2;
			break;
			case "H3":	multi = 3;
			break;
			case "H4":	multi = 4;
			break;
			case "H6":	multi = 6;
			break;
			case "H8":	multi = 8;
			break;
			case "H12":	multi = 12;
			break;
			case "D1":	multi = 24;
			break;
			case "M1":	multi = 0.016666;
			break;
			case "M3":	multi = 0.05;
			break;
			case "M5":	multi = 0.12;
			break;
			case "M15":	multi = 0.25;
			break;	
			case "M30":	multi = 0.5;
			break;	
		}

		for(int i = 0; i<ordersList.size(); i++) {

			
			Order order = ordersList.get(i);

			if(order.isFilledOrder()) {
							
				///hoursTrades += (order.BarsInTrade*multi);;
				hoursTrades += (order.BarsInTrade);

			}
		}

		return round2(SQUtils.safeDivide(hoursTrades*multi, ordersList.size()));
		//return round2(SQUtils.safeDivide(hoursTrades, ordersList.size()));
	}	
}