package SQ.Columns.Databanks;

import com.strategyquant.lib.*;
import com.strategyquant.datalib.*;
import com.strategyquant.tradinglib.*;

public class CochranFormula extends DatabankColumn {
    
	public CochranFormula() {
		super("CochranFormula", 
          DatabankColumn.Decimal2, // value display format
          ValueTypes.Maximize, // whether value should be maximized / minimized / approximated to a value   
          0, // target value if approximation was chosen  
          0, // average minimum of this value
          100); // average maximum of this value
		
    setWidth(80); // defaultcolumn width in pixels
    
		setTooltip("Cochran Formula - Numero di trade per campionamento Ligth");  
    
    /* If this new column is dependent on some other columns that have to vbe computed first, put them here.
       Make sure you don't creat circular dependency, such as A depends on B and B depends on A.
       Columns (=stats values) are identified by the name of class)
    */
    setDependencies("NumberOfProfits", "NumberOfLosses", "TotalDataDays");
	}
  
  //------------------------------------------------------------------------

  /**
   * This method should return computed value of this new column. You should typically compute it from the list of orders 
   * or from some already computed statistical values (other databank columns). 
   */
	@Override
	public double compute(SQStats stats, StatsTypeCombination combination, OrdersList ordersList, SettingsMap settings, SQStats statsLong, SQStats statsShort) throws Exception {
    
    /* an example - this is how you can get other values that this new value is dependent on */
    //int numberOfTrades = stats.getInt("NumberOfTrades");
    //double drawdownPct = stats.getDouble("DrawdownPct");
    int countProfits = stats.getInt("NumberOfProfits");
		int countLosses = stats.getInt("NumberOfLosses");
    int totaldays = stats.getInt("TotalDataDays");

		double WinPct = SQUtils.round2(SQUtils.safeDivide(countProfits, countProfits+countLosses));
    double n0 = round2((Math.pow(1.96,2.0)*WinPct*(1-WinPct))/Math.pow(0.05,2.0));

    /* round and return the value. It will be saved into stats under the key "CochranFormula" */
    //return round2((Math.pow(1.96,2.0)*WinPct*(1-WinPct))/Math.pow(0.05,2.0));
    
		return round2(n0/(1+((n0-1)/totaldays)));
	}	
}