/**
 * Authors: see task: https://roadmap.strategyquant.com/tasks/sq4_3361
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 */
package SQ.Columns.Databanks;

import com.strategyquant.lib.*;
import com.strategyquant.datalib.*;
import com.strategyquant.tradinglib.*;

public class GrossLossOOSISRatio extends DatabankColumn {

	public GrossLossOOSISRatio() {
		super("Gross Loss OOS/IS Ratio", 
				DatabankColumn.Decimal4, // value display format
				ValueTypes.Maximize, // whether value should be maximized / minimized / approximated to a value   
				0, // target value if approximation was chosen  
				-100, // average minimum of this value
				100); // average maximum of this value

		setWidth(80); // default column width in pixels

		setTooltip("Gross Loss OOS/IS Ratio");  

		setDependencies("GrossLoss");
	}

	//------------------------------------------------------------------------

	@Override
	public double getNumericValue(ResultsGroup results, String resultKey, byte direction, byte plType, byte sampleType) throws Exception {

		String stat = "GrossLoss";
		SQStats statsIS = getStats(results, resultKey, direction, plType, SampleTypes.InSample);
		SQStats statsOOS = getStats(results, resultKey, direction, plType, SampleTypes.OutOfSample);
		
		double resIS = statsIS.getDouble(stat);
		double resOOS = statsOOS.getDouble(stat);
		double res  =  SQUtils.safeDivide(resOOS, resIS);

		return res;
	}
	
	@Override
	public String getValue(ResultsGroup results, String resultKey, byte direction, byte plType, byte sampleType) throws Exception {	
		return this.getNumericValue(results, resultKey, direction, plType, sampleType) + "";
	}
}