package SQ.Columns.Databanks;
import com.strategyquant.tradinglib.Order;
import com.strategyquant.lib.L;
import com.strategyquant.lib.SettingsMap;
import com.strategyquant.tradinglib.DatabankColumn;
import com.strategyquant.tradinglib.OrdersList;
import com.strategyquant.tradinglib.SQStats;
import com.strategyquant.tradinglib.StatsTypeCombination;
import com.strategyquant.tradinglib.ValueTypes;
import com.strategyquant.lib.HistoryDataLoader;
import com.strategyquant.tradinglib.results.SpecialValues;
import com.strategyquant.tradinglib.ResultsGroup;
import com.strategyquant.tradinglib.Result;
import com.strategyquant.lib.SQTime;
import com.strategyquant.lib.HistoryOHLCData;
import com.strategyquant.datalib.session.*;
import java.util.Random;
import com.strategyquant.tradinglib.Directions;
import com.strategyquant.tradinglib.StatsTypeCombination;
import com.strategyquant.tradinglib.PlTypes;
import com.strategyquant.tradinglib.SampleTypes;
import it.unimi.dsi.fastutil.doubles.DoubleArrayList;

public class InternalProfitFactor extends DatabankColumn {
    
	public InternalProfitFactor() {
		super("Internal Profit Factor", 
          DatabankColumn.Decimal2, // value display format
          ValueTypes.Maximize, // whether value should be maximized / minimized / approximated to a value   
          0, // target value if approximation was chosen  
          0, // average minimum of this value
          100); // average maximum of this value

    setTooltip(L.tsq("Internal Profit Factor"));
		
    setWidth(80); // defaultcolumn width in pixels
    
	}
  
  //------------------------------------------------------------------------

  /**
   * This method should return computed value of this new column. You should typically compute it from the list of orders 
   * or from some already computed statistical values (other databank columns). 
   */
 @Override
	public double  compute(SQStats stats, StatsTypeCombination combination, OrdersList ordersList, SettingsMap settings, SQStats statsLong, SQStats statsShort, Result result, SettingsMap rgSpecialValues) throws Exception {

		if(result == null) {

			return 0;
		}
		ResultsGroup rg =  result.getResultsGroup();
		String symbol = rg.mainResult().getString(SpecialValues.Symbol, "N/A");
		String tf = rg.mainResult().getString(SpecialValues.Timeframe, "N/A");
		HistoryOHLCData data = null;
		try {
			HistoryDataLoader loader = new HistoryDataLoader();
			SettingsMap RGsettings = rg.specialValues();

			long from = RGsettings.getLong(SpecialValues.HistoryFrom);
			long to = RGsettings.getLong(SpecialValues.HistoryTo);

			/*HistoryOHLCData*/ data = loader.get(symbol, tf, from, to, Session.NoSession);
					}
		catch(Exception e) {
            Log.error("Exception", e);}
			
			Random rand = new Random(); 
			int dataLength = data.Time.length;	
			int orderListSize = ordersList.size();
			
			int c = 0;
			int closeBar = -1;
			int k = 0;
			int counter = 0;
			DoubleArrayList posPos = new DoubleArrayList();  
			DoubleArrayList posNeg = new DoubleArrayList();


		double pl = 0;  
		
		while (c < dataLength) {
		
			Order order = ordersList.get(k);
			
			long time = data.Time[c];
			float open = data.Open[c];
			float pOpen = 0;
			if(c>1) pOpen = data.Open[c-1];
			
			Order oOrder = null;

				if(combination.getDirection() ==0){

					int tempK = 0;
					
					if(k==0){
						
						tempK = 0;
						oOrder = ordersList.get(k);
					}					
					else if(k>0 && k<(ordersList.size()-1)){

						tempK = k-1;
						oOrder = ordersList.get(tempK);
					}
					else{

						tempK = k;
						oOrder = ordersList.get(tempK);
					} 
	
					if(c == closeBar){

					counter = 1;
					}

					pl = Math.log(open) - Math.log(pOpen);
															
					if(oOrder.isLong()&& c <=closeBar){
						

						if(pl>0){

							posPos.add(Math.abs(pl));
							
						} 
						else{

							posNeg.add(Math.abs(pl));
						} 
					}

					if(oOrder.isShort()&& c <=closeBar){


						if(pl<0){
							posPos.add(Math.abs(pl));
						} 
						else{

              posNeg.add(Math.abs(pl));
						} 
					}	
				}

				if(c <=closeBar){

					counter++;
					}		

	
				if(SQTime.toDateMinuteString(time).equals((SQTime.toDateMinuteString(order.OpenTime)))){

					closeBar = order.BarsInTrade+c;
					if(k<(orderListSize-1)) k++;
					
				}
			c++;			
		}
		
		double sumPos = 0;
		for (double p : posPos) sumPos += p;	
		double sumNeg = 0;
		for (double n : posNeg) sumNeg += n;
		double res = sumPos/sumNeg;
		return res;
		
	}




}
