/*
 * Copyright (c) 2017-2018, StrategyQuant - All rights reserved.
 *
 * Code in this file was made in a good faith that it is correct and does what it should.
 * If you found a bug in this code OR you have an improvement suggestion OR you want to include
 * your own code snippet into our standard library please contact us at:
 * https://roadmap.strategyquant.com
 *
 * This code can be used only within StrategyQuant products.
 * Every owner of valid (free, trial or commercial) license of any StrategyQuant product
 * is allowed to freely use, copy, modify or make derivative work of this code without limitations,
 * to be used in all StrategyQuant products and share his/her modifications or derivative work
 * with the StrategyQuant community.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 */
package SQ.Columns.Databanks;

import java.util.HashMap;

import com.strategyquant.lib.L;
import com.strategyquant.lib.SQUtils;
import com.strategyquant.lib.SettingsMap;
import com.strategyquant.lib.app.MainApp;
import com.strategyquant.tradinglib.*;

public class ProfitFactorMCRetestRatio extends DatabankColumn {
    
	public ProfitFactorMCRetestRatio() {
		super(L.tsq("Profit Factor / MC: Retest Profit Factor Ratio"), DatabankColumn.Decimal2, ValueTypes.Maximize, 0, -10000, 10000);

		setWidth(100);
		
		// this means that value depends on number of trading days 
		// and has to be normalized by days when comparing with another
		// result with different number of trading days
		setDependentOnTradingPeriod(true); 
		setDependencies("NetProfit");
		setDependencies("ProfitFactor");
	}
	
	/*
	 * celu compute mozes vyhodit. Takto to vyzera na pozadi ak ju nepretazis Override. Defaultne vracia 0
	 */
	@Override
	public double compute(SQStats stats, StatsTypeCombination combination, OrdersList ordersList, SettingsMap settings, SQStats statsLong, SQStats statsShort) throws Exception {
		return 0;
	}	
	
	//------------------------------------------------------------------------
	/*
	@Override
	public double getNumericValue(ResultsGroup results, String resultKey, byte direction, byte plType, byte sampleType) throws Exception {				
		SQStats statsOrigo = getStats(results, resultKey, direction, plType, sampleType);
		
		Result result = results.mainResult();
		HashMap<Integer, SQStats> statsForConfidenceLevels = (HashMap<Integer, SQStats>) result.get("MonteCarloManipulation"+"_ConfidenceLevelsStats");
		SQStats mc95Stats = statsForConfidenceLevels.get(95);
		
		return SQUtils.safeDivide(mc95Stats.getDouble("ProfitFactor"), statsOrigo.getDouble("ProfitFactor"));
		
	}
	*/
	
	//------------------------------------------------------------------------
	@Override
	
	public String getValue(ResultsGroup results, String resultKey, byte direction, byte plType, byte sampleType) throws Exception {
		SQStats statsOrigo = getStats(results, resultKey, direction, plType, sampleType);
		
		Result result = results.mainResult();
		//HashMap<Integer, SQStats> statsForConfidenceLevels = (HashMap<Integer, SQStats>) result.get("MonteCarloManipulation"+"_ConfidenceLevelsStats");

		HashMap<Integer, SQStats> statsForConfidenceLevels = (HashMap<Integer, SQStats>) result.get("MonteCarloRetest"+"_ConfidenceLevelsStats");


		SQStats mc95Stats = statsForConfidenceLevels.get(95);
	
		return SQUtils.safeDivide(mc95Stats.getDouble("ProfitFactor"), statsOrigo.getDouble("ProfitFactor"))+"";
	}
	
    }