package SQ.WhatIf;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.strategyquant.lib.SQTime;
import com.strategyquant.tradinglib.ClassConfig;
import com.strategyquant.tradinglib.Help;
import com.strategyquant.tradinglib.Order;
import com.strategyquant.tradinglib.OrdersList;
import com.strategyquant.tradinglib.Parameter;
import com.strategyquant.tradinglib.WhatIf;
import com.strategyquant.tradinglib.results.stats.comparator.OrderComparatorByOpenTime;

import it.unimi.dsi.fastutil.objects.ObjectListIterator;

@ClassConfig(name="(PFT) Profit Factor Trading", display="Profit Factor Trading (#PFPeriod#,#PFTreshold#)")
@Help("Help text")
public class ProfitFactorTrading extends WhatIf {
	public static final Logger Log = LoggerFactory.getLogger(ProfitFactorTrading.class);
	
	
	@Parameter(name="PFPeriod", defaultValue="10", minValue=0, maxValue=100000, step=1)
	public int PFPeriod;

	@Parameter(name="PFTreshold", defaultValue="0.5", minValue=0, maxValue=5, step=0.01)
	public double PFTreshold;

	@Override
	public void filter(OrdersList orders) throws Exception {


		OrdersList newOL = new OrdersList("PassedTrades"); 

		int i,start;
		if(PFPeriod==0) start =2;
		else start = PFPeriod == 1?2:PFPeriod;
		double sumPos = 0;
		double sumNeg = 0;
		double pf = 0;

		for(i = start;i < orders.size()-1;i++){

			Order order = orders.get(i); 
	
			if(order.isBalanceOrder()) continue; 

			if(PFPeriod ==0){
				
					Order o = orders.get(i-1);
				
					if(o.PipsPL<0) sumNeg = sumNeg+ Math.abs(o.PipsPL);		
					if(o.PipsPL>0) sumPos = sumPos+ o.PipsPL;

				if(sumNeg==0) {pf = 5d;}
				else pf = sumPos/sumNeg;
			}
			else{

				for(int k =1;k<=PFPeriod;k++){

					Order o = orders.get(i-k);
				
					if(o.PipsPL<0) sumNeg = sumNeg+ Math.abs(o.PipsPL);		
					if(o.PipsPL>0) sumPos = sumPos+ o.PipsPL;

				}

				if(sumNeg==0) {pf = 5d;}
				else pf = sumPos/sumNeg;
				
			}
				if(pf>PFTreshold){

					newOL.add(order);  
				}
			}
			// sorting newOL by positions open time
			newOL.sort(new OrderComparatorByOpenTime()); 
			
			orders.replaceWithList​(newOL); 
			
		}
	
}